<?php

/**
 * Vvveb
 *
 * Copyright (C) 2022  Ziadin Givan
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 *
 */

/**
 * Abstract class of editor plugins.
 *
 * @author Naoki Sawada
 */
class elFinderEditor {
	/**
	 * Array of allowed method by request from client side.
	 *
	 * @var array
	 */
	protected $allowed = [];

	/**
	 * elFinder instance.
	 *
	 * @var object elFinder instance
	 */
	protected $elfinder;

	/**
	 * Arguments.
	 *
	 * @var array argValues
	 */
	protected $args;

	/**
	 * Constructor.
	 *
	 * @param object $elfinder
	 * @param array  $args
	 */
	public function __construct($elfinder, $args) {
		$this->elfinder = $elfinder;
		$this->args     = $args;
	}

	/**
	 * Return boolean that this plugin is enabled.
	 *
	 * @return bool
	 */
	public function enabled() {
		return true;
	}

	/**
	 * Return boolean that $name method is allowed.
	 *
	 * @param string $name
	 *
	 * @return bool
	 */
	public function isAllowedMethod($name) {
		$checker = array_flip($this->allowed);

		return isset($checker[$name]);
	}

	/**
	 * Return $this->args value of the key.
	 *
	 * @param      string $key   target key
	 * @param      string $empty empty value
	 *
	 * @return     mixed
	 */
	public function argValue($key, $empty = '') {
		return isset($this->args[$key]) ? $this->args[$key] : $empty;
	}
}
