<?php

/**
 * Vvveb
 *
 * Copyright (C) 2022  Ziadin Givan
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 *
 */

/**
 * elFinder Plugin Abstract.
 *
 * @package elfinder
 * @author  Naoki Sawada
 * @license New BSD
 */
class elFinderPlugin {
	/**
	 * This plugin's options.
	 *
	 * @var array
	 */
	protected $opts = [];

	/**
	 * Get current volume's options.
	 *
	 * @param object $volume
	 *
	 * @return array options
	 */
	protected function getCurrentOpts($volume) {
		$name = substr(get_class($this), 14); // remove "elFinderPlugin"
		$opts = $this->opts;

		if (is_object($volume)) {
			$volOpts = $volume->getOptionsPlugin($name);

			if (is_array($volOpts)) {
				$opts = array_merge($opts, $volOpts);
			}
		}

		return $opts;
	}

	/**
	 * Is enabled with options.
	 *
	 * @param array    $opts
	 * @param elFinder $elfinder
	 *
	 * @return boolean
	 */
	protected function iaEnabled($opts, $elfinder = null) {
		if (! $opts['enable']) {
			return false;
		}

		// check post var 'contentSaveId' to disable this plugin
		if ($elfinder && ! empty($opts['disableWithContentSaveId'])) {
			$session           = $elfinder->getSession();
			$urlContentSaveIds = $session->get('urlContentSaveIds', []);

			if (! empty(elFinder::$currentArgs['contentSaveId']) && ($contentSaveId = elFinder::$currentArgs['contentSaveId'])) {
				if (! empty($urlContentSaveIds[$contentSaveId])) {
					$elfinder->removeUrlContentSaveId($contentSaveId);

					return false;
				}
			}
		}

		if (isset($opts['onDropWith']) && ! is_null($opts['onDropWith'])) {
			// plugin disabled by default, enabled only if given key is pressed
			if (isset($_REQUEST['dropWith']) && $_REQUEST['dropWith']) {
				$onDropWith = $opts['onDropWith'];
				$action     = (int)$_REQUEST['dropWith'];

				if (! is_array($onDropWith)) {
					$onDropWith = [$onDropWith];
				}

				foreach ($onDropWith as $key) {
					$key = (int)$key;

					if (($action & $key) === $key) {
						return true;
					}
				}
			}

			return false;
		}

		if (isset($opts['offDropWith']) && ! is_null($opts['offDropWith']) && isset($_REQUEST['dropWith'])) {
			// plugin enabled by default, disabled only if given key is pressed
			$offDropWith = $opts['offDropWith'];
			$action      = (int)$_REQUEST['dropWith'];

			if (! is_array($offDropWith)) {
				$offDropWith = [$offDropWith];
			}
			$res = true;

			foreach ($offDropWith as $key) {
				$key = (int)$key;

				if ($key === 0) {
					if ($action === 0) {
						$res = false;

						break;
					}
				} else {
					if (($action & $key) === $key) {
						$res = false;

						break;
					}
				}
			}

			if (! $res) {
				return false;
			}
		}

		return true;
	}
}
