<?php

/**
 * Vvveb
 *
 * Copyright (C) 2022  Ziadin Givan
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 *
 */

/**
 * elFinder - file manager for web.
 * Session Wrapper Interface.
 *
 * @package elfinder
 * @author  Naoki Sawada
 **/
interface elFinderSessionInterface {
	/**
	 * Session start.
	 *
	 * @return  self
	 **/
	public function start();

	/**
	 * Session write & close.
	 *
	 * @return  self
	 **/
	public function close();

	/**
	 * Get session data
	 * This method must be equipped with an automatic start / close.
	 *
	 * @param   string $key   Target key
	 * @param   mixed  $empty Return value of if session target key does not exist
	 *
	 * @return  mixed
	 **/
	public function get($key, $empty = '');

	/**
	 * Set session data
	 * This method must be equipped with an automatic start / close.
	 *
	 * @param   string $key  Target key
	 * @param   mixed  $data Value
	 *
	 * @return  self
	 **/
	public function set($key, $data);

	/**
	 * Get session data.
	 *
	 * @param   string $key Target key
	 *
	 * @return  self
	 **/
	public function remove($key);
}
